;;;
;;; Copyright (C) 2010 Frederic Vecoven
;;;
;;; This file is part of NixieClock
;;;
;;; NixieClock is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; NixieClock is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;;
	
;
; High-Voltage controller
;
	include hv.inc
	
	; variables
	extern	Vout, PFlag, BFlag, GFlag, ed_mode
	
	; delay.asm
	extern	delay_1ms
	
	; main.asm
	extern	init_tris

	; clock.asm
	extern	timer1_work
	
	; program.asm
	extern	init_ptrs
	

hv_code	CODE

;
; power initialization. (Interrupt must be enabled !)
;
init_power
	GLOBAL	init_power
	btfss	POWER_PIN
	bra	power_on		; 0 = main power ok
	bsf	PFlag,POWER_LOST	; 1 = no main power
	bra	power_off


;
; interrupt 0 : triggered when power goes down
;
int_power
	GLOBAL	int_power
	bcf	INTCON,INT0IF
	bsf	PFlag,POWER_LOST
	call	power_off
	return	

	
;
; check power : verify power state and call power_on once
; power is restored after a power outage.
;
; there is obviously one "ON" state, but there are different
; "OFF" states :
; - power outage :           POWER_ON = 0   POWER_LOST = 1
; - power off from program : POWER_ON = 0   POWER_LOST = 0  POWER_USER_OFF = 0
; - power off by user      : POWER_ON = 0   POWER_LOST = 0  POWER_USER_OFF = 1
;
; this function returns only when power is restored.
;
; timer1 work is still done, because the clock program must continue to run,
; in order to detect when the power should be restored (if it was shutdown by
; the clock program)
;
check_power
	GLOBAL	check_power
	btfsc	GFlag,TIMER1_WORK	; continue to do timer1 work...
	call	timer1_work
	btfss	PFlag,POWER_ON		; is power ON ?
	bra	check_outage		; no... check for outage
	bcf	PFlag,POWERING_ON	; yes.. reset powering_on and return
	return	
check_outage
	btfss	PFlag,POWER_LOST	;   is it due to a power outage ?
	bra	check_to_on		;   no... check if request to go ON
					;   yes.. 
	btfss	POWER_PIN		;     check if main power is back
	bra	verify_power		;     yes.. double-check
stay_off				;     no...
	bcf	OSCCON,IDLEN
	sleep				; sleep
	nop
	nop
	bra	check_power		; awaken... check again.
verify_power
	call	delay_1ms		; wait 
	btfsc	POWER_PIN		; and check power again
	bra	stay_off		; off.. we had a glitch.
	bcf	PFlag,POWER_LOST	; on... power outage is over !
	call	power_on
	bra	check_power
check_to_on
	btfsc	PFlag,POWERING_ON	; program asked for power on ?
	bra	to_on
	tstfsz	BFlag			; any button pressed ?
	bra	user_on			; yes.. power on
	bra	stay_off		; no... go back to sleep
user_on
	bsf	PFlag,POWERING_ON
	bcf	PFlag,POWER_USER_OFF
to_on	call	power_on
	bra	check_power


;
; Power on : safely turn high-voltage. 
;
power_on
	GLOBAL	power_on
	call	init_tris
	bcf	ANODE0
	bcf	ANODE1
	call	init_hv
	call	hv_on			; start HV
	call	init_ptrs		; recompute pointers
	bsf	T0CON,TMR0ON		; start timer0 (tube refresh)
	bsf	GFlag,TUBE_UPDATE	; force rebuild of tube data
	bcf	INTCON,INT0IF
	bsf	INTCON,INT0IE		; re-enable power outage detection
	bcf	PFlag,POWERING_ON	; no need to power on anymore
	bsf	PFlag,POWER_ON		; we are on
	clrf	BFlag			; ignore button pressed during off
	clrf	ed_mode			; reset editor mode
	return

;
; Power off : this function is called when a power outage is 
; detected. The high-voltage is turned off, timer 0 is stopped,
; and ports are set to input.
;
power_off
	GLOBAL	power_off
	bcf	PFlag,POWER_ON
	bcf	INTCON,INT0IE
	call	hv_off			; stop HV
	bcf	T0CON,TMR0ON		; stop timer0 (tube refresh)
	movlw	0xFF
	movwf	TRISA
	movwf	TRISB
	movwf	TRISC
	return
	

;
; init hv :
; - variable Vout is cleared
; - A/D is configured
; - PWM is configured
; - TIMER3 is used to automatically trigger A/D
;
init_hv
	clrf	Vout
				; A/D converter
	movlw	0x0E
	movwf	ADCON1		; RA0 analog, rest digital
	movlw	0x02
	movwf	ADCON2		; Fosc/32, no delay
	clrf	ADCON0
	bsf	ADCON0,ADON
	bsf	PIE1,ADIE
	bsf	IPR1,ADIP	; high-priority interrupt
	
				; PWM
	bcf	PWM_LAT
	bcf	PWM_DIR
	movlw	PWM_PR2
	movwf	PR2
	clrf	CCPR1L
	movlw	0x0C
	movwf	CCP1CON
	movlw	T2PRE
	movwf	T2CON
	
				; TIMER3
	movlw	0x0B
	movwf	CCP2CON		; compare mode, special event (trigger AD)
	movlw	high(T3VALUE)
	movwf	CCPR2H
	movlw	low(T3VALUE)
	movwf	CCPR2L
	movlw	(0xC8 | T3PRE)	; 16 bits, CCP2 special, OFF
	movwf	T3CON
	
	return
	
;
; Turn high-voltage on. The Vout value is slowly moved to its
; target value, to ramp up progressively.
;
hv_on
	clrf	ADCON0			; select RA0
	bsf	ADCON0,ADON		; turn A/D on
	clrf	CCPR1L			; duty = 0
	clrf	Vout			; Vout = 0
	bsf	T3CON,TMR3ON		; start timer3 (AD trigger)
	bsf	T2CON,TMR2ON		; start timer2 (PWM)
hv_ramp
	movlw	VOUT_AD8		; target value
	cpfslt	Vout
	bra	ramp_done
	call	delay_1ms
	call	delay_1ms
	incf	Vout,f
	bra	hv_ramp
ramp_done
	return
	
	
;
; turn off high-voltage
;
hv_off
	bcf	INTCON,GIEL
	bcf	INTCON,GIEH
	clrf	Vout
	bcf	T3CON,TMR3ON
	bcf	T2CON,TMR2ON
	bcf	ADCON0,GO
	bcf	ADCON0,ADON
	clrf	CCPR1L
	bcf	PIR1,ADIF
	bsf	INTCON,GIEH
	bsf	INTCON,GIEL
	return


;
; A/D interrupt : a conversion is finished. Adjust PWM duty.
;	
int_ad
	GLOBAL	int_ad
	bcf	PIR1,ADIF
	movf	Vout,w
	cpfsgt	ADRESH
	bra	too_low
too_high
	movf	CCPR1L,w
	btfss	STATUS,Z
	decf	CCPR1L,f
	return
too_low
	movlw	PWM_DUTY>>2
	cpfsgt	CCPR1L
	incf	CCPR1L,f
	return


	END
	
	