;;;
;;; Copyright (C) 2010 Frederic Vecoven
;;;
;;; This file is part of NixieClock
;;;
;;; NixieClock is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; NixieClock is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;;

;
; HV supply (PIC controlled)
;

	list p=18f2321
	include p18f2321.inc


XTAL_OSC	EQU	.32000000

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; High Voltage generation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
COIL		EQU	.330		; in uH
COIL_AMP	EQU	.500		; in mA
VIN		EQU	.12		; in V
VOUT		EQU	.170		; in V

AD_EVERY_US	EQU	.500		; sample High Voltage every x us.

; voltage divider (HV -> AD)
R1		EQU	.220000
R2		EQU	.3300

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Tube refresh rate
;
; Timer0 is used, in 1:128 prescale mode. With 32MHz clock, we get a nice
; refresh rate of ~120Hz by using timer0 in 8 bits mode.
;
; 1 tick of timer0 = 128 instructions = 128/8 = 16 us
; 256 ticks of timer0 = 16us x 256 = 4096us = 4ms, so ~122Hz refresh rate
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; Compute the threshold to compare with the A/D converter.
; We multiply by 1024 (10 bits AD), and scale with the voltage divider.
; Then we divide by the supply of the PIC. It is *not* 5V because there is
; a 1N4148 diode between VCC and VBAT, so we lose .6V. Finally, we divide
; by 4 to have a 8-bits value to compare against ADRESH.
VOUT_AD10	EQU	((VOUT*.1024*R2/(R1+R2))*.100)/.440
VOUT_AD8	EQU	VOUT_AD10>>2

Tcharge		EQU	(COIL_AMP * COIL)/VIN		; in ns
Tfall		EQU	(COIL_AMP * COIL)/(VOUT-VIN)	; in ns
Tflow		EQU	Tcharge + Tfall
Period		EQU	.4*Tflow/.3			; in ns


PWM_PRESCALE	EQU	.1
PWM_PR2		EQU	(((XTAL_OSC/(.4000*PWM_PRESCALE))*Period)/.1000000)-.1
PWM_DUTY	EQU	(Tcharge*(XTAL_OSC/(.1000*PWM_PRESCALE)))/.1000000


INST_PER_MS	EQU	XTAL_OSC/.4000
INST_EVERY_AD	EQU	(INST_PER_MS*AD_EVERY_US)/.1000
T3PRESCALE	EQU	.1
T3VALUE		EQU	(INST_EVERY_AD/T3PRESCALE)





#if (T3VALUE > 0xFFFF)
	error "CCPR2 value too big. Change timer3 prescale"
#endif

#if (PWM_PR2 > 0xFF)
	error "PR2 too big. Change timer2 PRESCALE"
#endif

#if (PWM_PRESCALE == .1)
T2PRE		EQU	0x00
#else
#if (PWM_PRESCALE == .4)
T2PRE		EQU	0x01
#else
#if (PWM_PRESCALE == .16)
T2PRE		EQU	0x02
#else
	error "Invalid PRESCALE for TIMER2"
#endif
#endif
#endif

#if (T3PRESCALE == .1)
T3PRE		EQU	0x00
#else
#if (T3PRESCALE == .2)
T3PRE		EQU	0x10
#else
#if (T3PRESCALE == .4)
T3PRE		EQU	0x20
#else
#if (T3PRESCALE == .8)
T3PRE		EQU	0x30
#else
	error "Invalid PRESCALE for TIMER3"
#endif
#endif
#endif
#endif


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Definitions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; number of tubes
#define	NUM_TUBES	4

; minimum brightness (must be >= 1)
#define	BRIGHT_MIN	2

; GFlag bits : generic flags
#define	TUBE_PAIR	1  	; indicates which pair of tubes is active
#define	TUBE_UPDATE	2	; true if data for timer0 should be rebuilt
#define	UPDATE_TFLAG	3	; if set, TFlags is updated
#define MANUAL_UPDATE	4	; true if the program should not be run
#define TIMER1_WORK	5	; true if timer1 has posted work to do

; PFlag bits (power flags)
#define POWER_ON	0	; set when power is ON
#define POWER_LOST	1	; set when power outage is ongoing
#define POWERING_ON	2	; set when power should be turned on
#define POWER_USER_OFF	3	; set when user powered the clock off

; BFlag bits (button flags)
#define	B0_SHORT	0
#define B0_LONG		1
#define B1_SHORT	2
#define B1_LONG		3
#define B_BOTH		4

; TFlag bits (time flags) : bit is set when a period of time has elapsed
#define	E_SEC		0
#define	E_MIN		1
#define	E_HOUR		2
#define	E_DAY		3
#define	E_WEEK		4
#define	E_MONTH		5
#define	E_YEAR		6

; CFlag bits (option flags). Clock display options
#define MODE12_BIT		0	; true = 12h mode    false = 24h mode
#define ZERO_SUPPRESS_BIT	1	; true = suppress leading 0
#define DATE_MMDD_BIT		2	; true = MMDD     false = DDMM

; auto_flags bits (one flag byte per tube)
#define	BRIGHT_CYCLE	0	; 1 = decrement   0 = increment

; offset in time variable
#define t_sec		time+0x0
#define t_min		time+0x1
#define t_hour		time+0x2
#define t_dow		time+0x3
#define t_day		time+0x4
#define t_month		time+0x5
#define t_year		time+0x6

#define t2_sec		time+0x7
#define t2_min		time+0x8
#define t2_hour		time+0x9
#define t2_dow		time+0xa
#define t2_day		time+0xb
#define t2_month	time+0xc
#define t2_year		time+0xd
	


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Macro
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
UNBCD	MACRO	reg
	swapf	reg,w
	andlw	0x0F		; W = tens
	rlncf	WREG,w		; W = 2 * tens
	subwf	reg,f		; 16 * tens + ones - 2 * tens
	subwf	reg,f		; 14 * tens + ones - 2 * tens
	subwf	reg,f		; 12 * tens + ones - 2 * tens
	movf	reg,w
	ENDM

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; PIN assigments
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#define	PWM_DIR		TRISC,2
#define	PWM_LAT		LATC,2
#define	PWM_PIN		PORTC,2

#define	ANODE0_DIR	TRISA,3
#define	ANODE0		LATA,3
#define	ANODE0_PIN	PORTA,3

#define	ANODE1_DIR	TRISC,3
#define	ANODE1		LATC,3
#define	ANODE1_PIN	PORTC,3

#define	POWER_DIR	TRISB,0
#define	POWER_PIN	PORTB,0

#define BUTTON0_DIR	TRISA,4
#define BUTTON0_PIN	PORTA,4

#define BUTTON1_DIR	TRISA,5
#define BUTTON1_PIN	PORTA,5
